# Copyright (C) 2014 Okami, okami@fuzetsu.info

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

import base64

from django.test.client import Client


class DAVClient(Client):
    http_authorization = None

    def http_auth(self, username, password):
        auth = '%s:%s' % (username, password)
        self.http_authorization = 'Basic %s' % (
            base64.b64encode(auth.encode())).decode()

    def dispatch(self, method, path, data='', content_type='text/xml',
                 follow=False, secure=False, **extra):
        meta = extra
        if self.http_authorization:
            meta['HTTP_AUTHORIZATION'] = self.http_authorization
        response = self.generic(
            method, path, data=data, content_type=content_type,
            secure=secure, **meta)
        if follow:
            response = self._handle_redirects(response, **meta)
        return response

    def propfind(self, path, data='', content_type='text/xml',
                 follow=False, secure=False, **extra):
        return self.dispatch('PROPFIND', path,
                             data=data, content_type=content_type,
                             follow=follow, secure=secure, **extra)

    def report(self, path, data='', content_type='text/xml',
               follow=False, secure=False, **extra):
        return self.dispatch('REPORT', path,
                             data=data, content_type=content_type,
                             follow=follow, secure=secure, **extra)

    def put(self, path, data='', content_type='text/vcard',
            follow=False, secure=False, **extra):
        return self.dispatch('PUT', path,
                             data=data, content_type=content_type,
                             follow=follow, secure=secure, **extra)

    def get(self, path, data='', content_type='text/xml',
            follow=False, secure=False, **extra):
        return self.dispatch('GET', path,
                             data=data, content_type=content_type,
                             follow=follow, secure=secure, **extra)

    def delete(self, path, data='', content_type='text/xml',
               follow=False, secure=False, **extra):
        return self.dispatch('DELETE', path,
                             data=data, content_type=content_type,
                             follow=follow, secure=secure, **extra)



# Copyright (C) 2022 Kyoken, kyoken@kyoken.ninja

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

import logging

import radicale
import radicale.config

from django.conf import settings


def config_load(*args, **kwargs):
    configuration = radicale.config.Configuration(radicale.config.DEFAULT_CONFIG_SCHEMA)
    configuration.update(settings.DJRADICALE_CONFIG)
    return configuration


radicale.config.load = config_load

default_app_config = 'djradicale.config.DjRadicaleConfig'



# Copyright (C) 2014 Okami, okami@fuzetsu.info

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from django.contrib.auth.models import User
from django.urls import reverse
from django.test import TestCase

from djradicale.models import DBCollection, DBProperties

from . import DAVClient


class DjRadicaleTestCase(TestCase):
    DATA_CASES = {
        # PROPFIND_1 ##########################################################
        'PROPFIND_1': {
            'request': '''<?xml version="1.0"?>
<D:propfind xmlns:D="DAV:" xmlns:x0="http://calendarserver.org/ns/">
    <D:prop>
        <D:resourcetype/>
        <D:supported-report-set/>
        <x0:getctag/>
    </D:prop>
</D:propfind>
''',
            'response': '''<?xml version='1.0' encoding='utf-8'?>
<multistatus xmlns="DAV:" xmlns:CR="urn:ietf:params:xml:ns:carddav" xmlns:CS="http://calendarserver.org/ns/"><response><href>/radicale/user/addressbook.vcf/</href><propstat><prop><resourcetype><CR:addressbook /><collection /></resourcetype><supported-report-set><supported-report><report><expand-property /></report></supported-report><supported-report><report><principal-search-property-set /></report></supported-report><supported-report><report><principal-property-search /></report></supported-report><supported-report><report><sync-collection /></report></supported-report><supported-report><report><CR:addressbook-multiget /></report></supported-report><supported-report><report><CR:addressbook-query /></report></supported-report></supported-report-set><CS:getctag>"2e450f7001eaf27c082934cbbd5dc65104b2778f3b582e4aa0341983cb2d5b3b"</CS:getctag></prop><status>HTTP/1.1 200 OK</status></propstat></response></multistatus>''',
        },
        # PROPFIND_2 ##########################################################
        'PROPFIND_2': {
            'request': '''<?xml version="1.0" encoding="utf-8" ?>
<D:propfind xmlns:D="DAV:">
    <D:prop>
        <D:displayname/>
        <D:getetag/>
        <D:getcontenttype/>
    </D:prop>
</D:propfind>
''',
            'response': '''<?xml version='1.0' encoding='utf-8'?>
<multistatus xmlns="DAV:"><response><href>/radicale/user/addressbook.vcf/</href><propstat><prop><displayname>user/addressbook.vcf</displayname><getetag>"2e450f7001eaf27c082934cbbd5dc65104b2778f3b582e4aa0341983cb2d5b3b"</getetag><getcontenttype>text/vcard</getcontenttype></prop><status>HTTP/1.1 200 OK</status></propstat></response></multistatus>''',
        },
        # PUT #################################################################
        'PUT': {
            'request': '''BEGIN:VCARD
VERSION:3.0
PRODID:-//Inverse inc.//SOGo Connector 1.0//EN
UID:test.vcf
N:a;a
FN:{fn}
TEL;TYPE=cell:+1234567890
X-MOZILLA-HTML:FALSE
END:VCARD
''',
            'response': '',
        },
        # GET #################################################################
        'GET': {
            'request': '',
            'response': '''BEGIN:VCARD
VERSION:3.0
PRODID:-//Inverse inc.//SOGo Connector 1.0//EN
UID:test.vcf
FN:{fn}
N:a;a;;;
TEL;TYPE=cell:+1234567890
X-MOZILLA-HTML:FALSE
END:VCARD
''',
        },
        # REPORT_EMPTY ########################################################
        'REPORT_EMPTY': {
            'request': '''<?xml version="1.0" encoding="utf-8" ?>
<C:addressbook-multiget xmlns:C="urn:ietf:params:xml:ns:carddav">
    <D:prop xmlns:D="DAV:">
        <D:displayname/>
        <D:getetag/>
        <C:address-data/>
    </D:prop>
    <D:href xmlns:D="DAV:">{path}</D:href>
</C:addressbook-multiget>
''',
            'response': '''<?xml version='1.0' encoding='utf-8'?>
<multistatus xmlns="DAV:" />''',
        },
        # REPORT ##############################################################
        'REPORT': {
            'request': '''<?xml version="1.0" encoding="utf-8" ?>
<C:addressbook-multiget xmlns:C="urn:ietf:params:xml:ns:carddav">
    <D:prop xmlns:D="DAV:">
        <D:displayname/>
        <D:getetag/>
        <C:address-data/>
    </D:prop>
    <D:href xmlns:D="DAV:">{path}</D:href>
</C:addressbook-multiget>
''',
            'response': '''<?xml version='1.0' encoding='utf-8'?>
<multistatus xmlns="DAV:" xmlns:CR="urn:ietf:params:xml:ns:carddav"><response><href>{path}test.vcf</href><propstat><prop><getetag>"{etag}"</getetag><CR:address-data>BEGIN:VCARD
VERSION:3.0
PRODID:-//Inverse inc.//SOGo Connector 1.0//EN
UID:test.vcf
FN:{fn}
N:a;a;;;
TEL;TYPE=cell:+1234567890
X-MOZILLA-HTML:FALSE
END:VCARD
</CR:address-data></prop><status>HTTP/1.1 200 OK</status></propstat><propstat><prop><displayname /></prop><status>HTTP/1.1 404 Not Found</status></propstat></response></multistatus>''',
        },
        # DELETE ##############################################################
        'DELETE': {
            'request': '',
            'response': '''<?xml version='1.0' encoding='utf-8'?>
<multistatus xmlns="DAV:"><response><href>{path}</href><status>HTTP/1.1 200 OK</status></response></multistatus>''',
        },
    }

    maxDiff = None

    def setUp(self):
        User.objects.create_user(username='user', password='password')
#        DBCollection.objects.create(path="user")
        DBCollection.objects.create(path="user/addressbook.vcf", parent_path="user")
        DBProperties.objects.create(path="user/addressbook.vcf", text='{"tag": "VADDRESSBOOK"}')
        self.client = DAVClient()

    def propfind_1_anonymous(self):
        path = reverse('djradicale:application', kwargs={
            'url': 'user/addressbook.vcf/',
        })
        response = self.client.propfind(
            path, data=self.DATA_CASES['PROPFIND_1']['request'].format(**{
                'path': path,
            }))
        self.assertEqual(response.status_code, 401)

    def propfind_1(self):
        path = reverse('djradicale:application', kwargs={
            'url': 'user/addressbook.vcf/',
        })
        response = self.client.propfind(
            path, data=self.DATA_CASES['PROPFIND_1']['request'].format(**{
                'path': path,
            }))
        self.assertEqual(response.status_code, 207)
        self.assertEqual(
            response.content.decode(),
            self.DATA_CASES['PROPFIND_1']['response'].format(**{
                'path': path,
            }))

    def propfind_2(self):
        path = reverse('djradicale:application', kwargs={
            'url': 'user/addressbook.vcf/',
        })
        response = self.client.propfind(
            path, data=self.DATA_CASES['PROPFIND_2']['request'].format(**{
                'path': path,
            }))
        self.assertEqual(response.status_code, 207)
        self.assertEqual(
            response.content.decode(),
            self.DATA_CASES['PROPFIND_2']['response'].format(**{
                'path': path,
            }))

    def report_empty(self):
        path = reverse('djradicale:application', kwargs={
            'url': 'user/addressbook.vcf/',
        })
        response = self.client.report(
            path, data=self.DATA_CASES['REPORT_EMPTY']['request'].format(**{
                'path': path,
            }))
        self.assertEqual(response.status_code, 207)
        self.assertEqual(
            response.content.decode(),
            self.DATA_CASES['REPORT_EMPTY']['response'].format(**{
                'path': path,
            }))

    def report(self, fn, etag):
        path = reverse('djradicale:application', kwargs={
            'url': 'user/addressbook.vcf/',
        })
        response = self.client.report(
            path, data=self.DATA_CASES['REPORT']['request'].format(**{
                'path': path,
            }))
        self.assertEqual(response.status_code, 207)
        self.assertEqual(
            response.content.decode().split(),
            self.DATA_CASES['REPORT']['response'].format(**{
                'path': path,
                'fn': fn,
                'etag': etag,
            }).split())

    def put(self, fn):
        path = reverse('djradicale:application', kwargs={
            'url': 'user/addressbook.vcf/test.vcf',
        })
        response = self.client.put(
            path, data=self.DATA_CASES['PUT']['request'].format(**{
                'path': path,
                'fn': fn,
            }))
        self.assertEqual(response.status_code, 201)
        self.assertEqual(
            response.content.decode(),
            self.DATA_CASES['PUT']['response'].format(**{
                'path': path,
                'fn': fn,
            }))

    def get(self, fn):
        path = reverse('djradicale:application', kwargs={
            'url': 'user/addressbook.vcf/test.vcf',
        })
        response = self.client.get(path)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.content.decode().split(),
            self.DATA_CASES['GET']['response'].format(**{
                'path': path,
                'fn': fn,
            }).split())

    def delete(self):
        path = reverse('djradicale:application', kwargs={
            'url': 'user/addressbook.vcf/test.vcf',
        })
        response = self.client.delete(path)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.content.decode(),
            self.DATA_CASES['DELETE']['response'].format(**{
                'path': path,
            }))

    def test_everything(self):
        self.propfind_1_anonymous()
        self.client.http_auth('user', 'password')
        self.propfind_1()
        self.propfind_2()
        self.report_empty()
        # create
        self.put(fn='John Smith')
        self.get(fn='John Smith')
        self.report(fn='John Smith', etag='9227dd9db5e16cff639191598273c84b20c8a53719e11a200be855c661e44799')
        # update
        self.put(fn='J.S.')
        self.get(fn='J.S.')
        self.report(fn='J.S.', etag='1ab6a4dd3221a09b3fe981f645d5f71f0e9dbcdff2a7343e2f498546b806dc9f')
        self.delete()
        self.report_empty()



# Copyright (C) 2014 Okami, okami@fuzetsu.info

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from django.apps import AppConfig


class DjRadicaleConfig(AppConfig):
    name = 'djradicale'
    verbose_name = 'Radicale'



# Generated by Django 2.0 on 2018-01-11 00:07

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='DBCollection',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('path', models.CharField(max_length=255, unique=True, verbose_name='Path')),
                ('parent_path', models.TextField(verbose_name='Parent Path')),
            ],
            options={
                'verbose_name': 'Collection',
                'verbose_name_plural': 'Collections',
                'db_table': 'djradicale_collection',
            },
        ),
        migrations.CreateModel(
            name='DBItem',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('text', models.TextField(verbose_name='Text')),
                ('timestamp', models.DateTimeField(auto_now=True, verbose_name='Timestamp')),
                ('collection', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='items', to='djradicale.DBCollection', verbose_name='Collection')),
            ],
            options={
                'verbose_name': 'Item',
                'verbose_name_plural': 'Items',
                'db_table': 'djradicale_item',
                'ordering': ('timestamp',),
            },
        ),
        migrations.CreateModel(
            name='DBProperties',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('path', models.CharField(max_length=255, unique=True, verbose_name='Path')),
                ('text', models.TextField(verbose_name='Text')),
            ],
            options={
                'verbose_name': 'Properties',
                'verbose_name_plural': 'Properties',
                'db_table': 'djradicale_properties',
            },
        ),
        migrations.AlterUniqueTogether(
            name='dbitem',
            unique_together={('name', 'collection')},
        ),
    ]



# Copyright (C) 2022 Kyoken, kyoken@kyoken.ninja

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from django import forms
from django.contrib import admin

from .models import DBCollection, DBItem, DBProperties


class DBCollectionForm(forms.ModelForm):
    class Meta(object):
        fields = 'path', 'parent_path'
        model = DBCollection
        widgets = {
            'path': forms.TextInput,
            'parent_path': forms.TextInput,
        }


class DBItemForm(forms.ModelForm):
    class Meta(object):
        fields = 'collection', 'name', 'text'
        model = DBItem
        widgets = {
            'name': forms.TextInput,
        }


class DBPropertiesForm(forms.ModelForm):
    class Meta(object):
        fields = 'path', 'text'
        model = DBProperties
        widgets = {
            'path': forms.TextInput,
        }


class DBCollectionAdmin(admin.ModelAdmin):
    form = DBCollectionForm
    fields = 'path', 'parent_path'
    list_display = 'path', 'tag', 'last_modified'


class DBItemAdmin(admin.ModelAdmin):
    form = DBItemForm
    fields = 'collection', 'name', 'text', 'timestamp'
    list_display = 'name', 'fn', 'collection', 'timestamp'
    list_filter = 'collection',
    readonly_fields = 'timestamp',


class DBPropertiesAdmin(admin.ModelAdmin):
    form = DBPropertiesForm
    fields = 'path', 'text'
    list_display = 'path', 'tag'


admin.site.register(DBCollection, DBCollectionAdmin)
admin.site.register(DBItem, DBItemAdmin)
admin.site.register(DBProperties, DBPropertiesAdmin)



# Copyright (C) 2014 Okami, okami@fuzetsu.info

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

import json
import re

from django.conf import settings
from django.urls import reverse
from django.db import models
from django.db.models import Q

from radicale.item import Item


class DBCollectionQuerySet(models.query.QuerySet):
    def as_collections(self):
        for c in self:
            yield c.as_collection()


class DBItemQuerySet(models.query.QuerySet):
    def as_items(self):
        for i in self:
            yield i.as_item()


class DBCollection(models.Model):
    """
    Table of collections.
    """
    objects = DBCollectionQuerySet.as_manager()
    path = models.CharField('Path', max_length=255, unique=True)
    parent_path = models.TextField('Parent Path')

    def get_absolute_url(self):
        return reverse('djradicale:application', kwargs={'url': self.path})

    @property
    def last_modified(self):
        if self.items.exists():
            return self.items.latest('timestamp').timestamp

    @property
    def tag(self):
        try:
            return DBProperties.objects.get(path=self.path).tag
        except DBProperties.DoesNotExist:
            pass

    def __str__(self):
        return self.path

    def __unicode__(self):
        return self.path

    def as_collection(self):
        from .storage import Collection
        return Collection(self.path)

    class Meta(object):
        db_table = 'djradicale_collection'
        verbose_name = 'Collection'
        verbose_name_plural = 'Collections'


class DBItem(models.Model):
    """
    Table of collection's items.
    """
    objects = DBItemQuerySet.as_manager()

    collection = models.ForeignKey(
        'DBCollection', verbose_name='Collection', related_name='items',
        on_delete=models.CASCADE)
    name = models.CharField('Name', max_length=255)
    text = models.TextField('Text')
    timestamp = models.DateTimeField('Timestamp', auto_now=True)

    def get_absolute_url(self):
        return reverse('djradicale:application', kwargs={
            'url': '%s/%s' % (self.collection.path, self.name),
        })

    def __str__(self):
        return self.name

    def __unicode__(self):
        return self.name

    def _get_field(self, field):
        for line in self.text.split('\n'):
            if line.startswith(field + ':'):
                return line[len(field + ':'):]

    def as_item(self):
        collection = self.collection.as_collection()
        return Item(
            collection=collection,
            collection_path=collection.path,
            href=self.name,
            text=self.text,
            last_modified=self.timestamp.timestamp
        )

    @property
    def fn(self):
        return self._get_field('FN')

    @property
    def path(self):
        return '%s/%s' % (self.collection.path, self.name)

    class Meta(object):
        db_table = 'djradicale_item'
        ordering = 'timestamp',
        verbose_name = 'Item'
        verbose_name_plural = 'Items'
        unique_together = 'name', 'collection'


class DBProperties(models.Model):
    """
    Table of collection's properties.
    """
    path = models.CharField('Path', max_length=255, unique=True)
    text = models.TextField('Text')

    @property
    def tag(self):
        if self.text:
            return json.loads(self.text).get('tag')

    class Meta(object):
        db_table = 'djradicale_properties'
        verbose_name = 'Properties'
        verbose_name_plural = 'Properties'



# Copyright (C) 2022 Kyoken, kyoken@kyoken.ninja

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

import base64
import copy
import logging

from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.http import HttpResponse
from django.urls import reverse
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import RedirectView, View

from radicale import Application, config, log


class ApplicationResponse(HttpResponse):
    def start_response(self, status, headers):
        self.status_code = int(status.split(' ')[0])
        for k, v in dict(headers).items():
            self[k] = v


class DjRadicaleView(Application, View):
    http_method_names = [
        'delete',
        'get',
        'head',
        'mkcalendar',
        'mkcol',
        'move',
        'options',
        'propfind',
        'proppatch',
        'put',
        'report',
    ]

    def __init__(self, **kwargs):
        configuration = config.load()
        Application.__init__(self, configuration)
        View.__init__(self, **kwargs)

    @method_decorator(csrf_exempt)
    def dispatch(self, request, *args, **kwargs):
        if not request.method.lower() in self.http_method_names:
            return self.http_method_not_allowed(request, *args, **kwargs)
        response = ApplicationResponse()

        path = request.META['PATH_INFO']
        if path.startswith(settings.DJRADICALE_PREFIX):
            # cut known prefix from path (PATH_INFO) and
            # move it into the base prefix (HTTP_X_SCRIPT_NAME)
            request.META['PATH_INFO'] = path[len(settings.DJRADICALE_PREFIX):]
            request.META['HTTP_X_SCRIPT_NAME'] = settings.DJRADICALE_PREFIX.rstrip('/')

        answer = self(request.META, response.start_response)
        for i in answer:
            response.write(i)
        return response


class WellKnownView(RedirectView):
    type = 'caldav'

    def get_redirect_url(self):
        return self.request.build_absolute_uri(settings.DJRADICALE_PREFIX)

    @method_decorator(csrf_exempt)
    def dispatch(self, request, *args, **kwargs):
        return self.get(request, *args, **kwargs)



# Copyright (C) 2014 Okami, okami@fuzetsu.info

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from django.contrib.auth.models import User
from django.urls import reverse
from django.test import TestCase

from djradicale.models import DBCollection, DBProperties

from . import DAVClient


class DjRadicaleTestCase(TestCase):
    DATA_CASES = {
        # PUT #################################################################
        'PUT': {
            'request': '''BEGIN:VCALENDAR
CALSCALE:GREGORIAN
PRODID:-//Ximian//NONSGML Evolution Calendar//EN
VERSION:2.0
BEGIN:VTIMEZONE
TZID:/freeassociation.sourceforge.net/Europe/Moscow
X-LIC-LOCATION:Europe/Moscow
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19300621T000000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19810401T000000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19811001T000000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19820401T000000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19821001T000000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19830401T000000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19831001T000000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19840401T000000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19840930T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19850331T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19850929T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19860330T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19860928T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19870329T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19870927T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19880327T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19880925T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19890326T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19890924T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19900325T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19900930T030000
TZOFFSETFROM:+0300
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:EEST
DTSTART:19910331T020000
TZOFFSETFROM:+0200
TZOFFSETTO:+0300
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:EET
DTSTART:19910929T030000
TZOFFSETFROM:+0300
TZOFFSETTO:+0200
END:STANDARD
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19920119T020000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19920329T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19920927T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19930328T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19930926T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19940327T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19940925T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19950326T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19950924T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19960331T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19961027T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19970330T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19971026T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19980329T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19981025T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19990328T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19991031T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:20000326T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20001029T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:20010325T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20011028T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:20020331T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20021027T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:20030330T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20031026T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:20040328T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20041031T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:20050327T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20051030T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:20060326T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20061029T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:20070325T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20071028T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:20080330T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20081026T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:20090329T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20091025T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:20100328T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20101031T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20110327T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:STANDARD
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20141026T020000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
END:VTIMEZONE
BEGIN:VEVENT
UID:20160729T101032Z-9313-1000-4291-10@localhost
DTSTAMP:20160729T063824Z
DTSTART;TZID=/freeassociation.sourceforge.net/Europe/Moscow:
 20160721T090000
DTEND;TZID=/freeassociation.sourceforge.net/Europe/Moscow:
 20160721T093000
SEQUENCE:2
SUMMARY:zzzzz
CLASS:PUBLIC
TRANSP:OPAQUE
CREATED:20160729T101116Z
LAST-MODIFIED:20160729T101116Z
END:VEVENT
END:VCALENDAR
''',
            'response': '',
        },
        # REPORT ##############################################################
        'REPORT': {
            'request': (
                '<?xml version="1.0" encoding="UTF-8"?>\n'
                '<C:calendar-multiget xmlns:D="DAV:" xmlns:C="urn:ietf:params:xml:ns:caldav">'
                '<D:prop><D:getetag/><C:calendar-data/></D:prop>'
                '<D:href>{url}</D:href></C:calendar-multiget>\n'
            ),
            'response': '''<?xml version='1.0' encoding='utf-8'?>
<multistatus xmlns="DAV:" xmlns:C="urn:ietf:params:xml:ns:caldav"><response><href>/radicale/user/calendar.ics/20160729T101032Z-9313-1000-4291-10_localhost-20160729T101116Z.ics</href><propstat><prop><getetag>"7e542ba33f6239c2fb6171ee1dec39c583e6462488c84c9b78e0361676ffb816"</getetag><C:calendar-data>BEGIN:VCALENDAR
VERSION:2.0
CALSCALE:GREGORIAN
PRODID:-//Ximian//NONSGML Evolution Calendar//EN
BEGIN:VTIMEZONE
TZID:/freeassociation.sourceforge.net/Europe/Moscow
BEGIN:STANDARD
DTSTART:19300621T000000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19811001T000000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19821001T000000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19831001T000000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19840930T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19850929T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19860928T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19870927T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19880925T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19890924T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19900930T030000
TZNAME:MSK
TZOFFSETFROM:+0300
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19910929T030000
TZNAME:EET
TZOFFSETFROM:+0300
TZOFFSETTO:+0200
END:STANDARD
BEGIN:STANDARD
DTSTART:19920119T020000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19920927T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19930926T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19940925T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19950924T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19961027T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19971026T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19981025T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19991031T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:20001029T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:20011028T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:20021027T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:20031026T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:20041031T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:20051030T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:20061029T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:20071028T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:20081026T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:20091025T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:20101031T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:20110327T020000
TZNAME:MSK
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:STANDARD
BEGIN:STANDARD
DTSTART:20141026T020000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
DTSTART:19810401T000000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19820401T000000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19830401T000000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19840401T000000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19850331T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19860330T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19870329T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19880327T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19890326T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19900325T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19910331T020000
TZNAME:EEST
TZOFFSETFROM:+0200
TZOFFSETTO:+0300
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19920329T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19930328T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19940327T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19950326T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19960331T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19970330T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19980329T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19990328T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:20000326T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:20010325T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:20020331T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:20030330T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:20040328T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:20050327T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:20060326T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:20070325T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:20080330T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:20090329T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:20100328T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
X-LIC-LOCATION:Europe/Moscow
END:VTIMEZONE
BEGIN:VEVENT
UID:20160729T101032Z-9313-1000-4291-10@localhost
DTSTART;TZID=/freeassociation.sourceforge.net/Europe/Moscow:20160721T090000
DTEND;TZID=/freeassociation.sourceforge.net/Europe/Moscow:20160721T093000
CLASS:PUBLIC
CREATED:20160729T101116Z
DTSTAMP:20160729T063824Z
LAST-MODIFIED:20160729T101116Z
SEQUENCE:2
SUMMARY:zzzzz
TRANSP:OPAQUE
END:VEVENT
END:VCALENDAR
</C:calendar-data></prop><status>HTTP/1.1 200 OK</status></propstat></response></multistatus>''',
        },
        # GET #################################################################
        'GET': {
            'request': '',
            'response': '''BEGIN:VCALENDAR
VERSION:2.0
CALSCALE:GREGORIAN
PRODID:-//Ximian//NONSGML Evolution Calendar//EN
BEGIN:VTIMEZONE
TZID:/freeassociation.sourceforge.net/Europe/Moscow
BEGIN:STANDARD
DTSTART:19300621T000000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19811001T000000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19821001T000000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19831001T000000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19840930T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19850929T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19860928T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19870927T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19880925T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19890924T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19900930T030000
TZNAME:MSK
TZOFFSETFROM:+0300
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19910929T030000
TZNAME:EET
TZOFFSETFROM:+0300
TZOFFSETTO:+0200
END:STANDARD
BEGIN:STANDARD
DTSTART:19920119T020000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19920927T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19930926T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19940925T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19950924T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19961027T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19971026T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19981025T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:19991031T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:20001029T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:20011028T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:20021027T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:20031026T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:20041031T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:20051030T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:20061029T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:20071028T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:20081026T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:20091025T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:20101031T030000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
DTSTART:20110327T020000
TZNAME:MSK
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:STANDARD
BEGIN:STANDARD
DTSTART:20141026T020000
TZNAME:MSK
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
DTSTART:19810401T000000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19820401T000000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19830401T000000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19840401T000000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19850331T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19860330T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19870329T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19880327T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19890326T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19900325T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19910331T020000
TZNAME:EEST
TZOFFSETFROM:+0200
TZOFFSETTO:+0300
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19920329T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19930328T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19940327T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19950326T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19960331T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19970330T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19980329T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:19990328T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:20000326T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:20010325T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:20020331T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:20030330T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:20040328T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:20050327T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:20060326T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:20070325T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:20080330T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:20090329T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:DAYLIGHT
DTSTART:20100328T020000
TZNAME:MSD
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
X-LIC-LOCATION:Europe/Moscow
END:VTIMEZONE
BEGIN:VEVENT
UID:20160729T101032Z-9313-1000-4291-10@localhost
DTSTART;TZID=/freeassociation.sourceforge.net/Europe/Moscow:20160721T090000
DTEND;TZID=/freeassociation.sourceforge.net/Europe/Moscow:20160721T093000
CLASS:PUBLIC
CREATED:20160729T101116Z
DTSTAMP:20160729T063824Z
LAST-MODIFIED:20160729T101116Z
SEQUENCE:2
SUMMARY:zzzzz
TRANSP:OPAQUE
END:VEVENT
END:VCALENDAR
''',
        },
    }

    maxDiff = None

    def setUp(self):
        User.objects.create_user(username='user', password='password')
        DBCollection.objects.create(path="user/calendar.ics", parent_path="user")
        DBProperties.objects.create(path="user/calendar.ics", text='{"tag": "VCALENDAR"}')
        self.client = DAVClient()

    def report(self, name):
        path = reverse('djradicale:application', kwargs={
            'url': 'user/calendar.ics/',
        })
        response = self.client.report(
            path, data=self.DATA_CASES['REPORT']['request'].format(**{
                'url': '{}{}'.format(path, name),
            }))
        self.assertEqual(response.status_code, 207)
        # split the strings to avoid lf/crlf problems
        self.assertEqual(
            response.content.decode().split(),
            self.DATA_CASES['REPORT']['response'].split())

    def put(self, name):
        path = reverse('djradicale:application', kwargs={
            'url': 'user/calendar.ics/{}'.format(name),
        })
        response = self.client.put(
            path, data=self.DATA_CASES['PUT']['request'].format(**{
                'name': name,
            }))
        self.assertEqual(response.status_code, 201)
        self.assertEqual(
            response.content.decode(),
            self.DATA_CASES['PUT']['response'])

    def get(self, name):
        path = reverse('djradicale:application', kwargs={
            'url': 'user/calendar.ics/{}'.format(name),
        })
        response = self.client.get(path)
        self.assertEqual(response.status_code, 200)
        # split the strings to avoid lf/crlf problems
        self.assertEqual(
            response.content.decode().split(),
            self.DATA_CASES['GET']['response'].split()
        )

    def test_everything(self):
        self.client.http_auth('user', 'password')
        # create
        name = '20160729T101032Z-9313-1000-4291-10_localhost-20160729T101116Z.ics'
        self.put(name=name)
        self.get(name=name)
        self.report(name=name)



# Copyright (C) 2022 Kyoken, kyoken@kyoken.ninja

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

import json
import os
import logging
import datetime

from contextlib import contextmanager

from django.conf import settings
from django.db import transaction
from django.db.models import Q

from radicale import types
from radicale.storage import BaseCollection, BaseStorage
from radicale.pathutils import strip_path, unstrip_path

from .models import DBCollection, DBItem, DBProperties

logger = logging.getLogger('djradicale')


class Collection(BaseCollection):
    def __init__(self, path, **kwargs):
        self._path = path

    @property
    def path(self):
        return self._path

    def get_all(self):
        q = Q(collection__path=self.path)
        return DBItem.objects.filter(q).as_items()

    def get_multi(self, hrefs):
        q = Q(collection__path=self.path, name__in=hrefs)
        for i in DBItem.objects.filter(q).as_items():
            yield (i.href, i)

    def has_uid(self, uid):
        q = Q(collection__path=self.path, name=uid)
        return DBItem.objects.filter(q).exists()

    def upload(self, href, item):
        try:
            dbcollection = DBCollection.objects.get(path=self.path)
            dbitem, _ = DBItem.objects.get_or_create(
                collection=dbcollection,
                name=href
            )
            dbitem.text = item.serialize()
            dbitem.save()
        except DBCollection.DoesNotExist:
            pass
        else:
            return item

    def delete(self, href=None):
        if href is None:
            DBItem.objects.filter(collection__path=self.path).delete()
            DBCollection.objects.filter(path=self.path).delete()
            DBProperties.objects.filter(path=self.path).delete()
        else:
            DBItem.objects.filter(collection__path=self.path, name=href).delete()

    def get_meta(self, key=None):
        try:
            p = DBProperties.objects.get(path=self.path)
            meta = json.loads(p.text)
            if key is None:
                return meta
            else:
                return meta.get(key)

        except DBProperties.DoesNotExist:
            if key is None:
                return {}

    def set_meta(self, props):
        p, created = DBProperties.objects.get_or_create(path=self.path)
        p.text = json.dumps(props)
        p.save()

    @property
    def last_modified(self):
        try:
            collection = DBCollection.objects.get(path=self.path)
        except DBCollection.DoesNotExist:
            pass
        else:
            if collection.last_modified:
                return datetime.datetime.strftime(
                    collection.last_modified, '%a, %d %b %Y %H:%M:%S %z')


class Storage(BaseStorage):
    def discover(self, path, depth='0'):
        stripped_path = strip_path(path)

        if stripped_path == '':
            yield Collection('')
            return

        for c in DBCollection.objects.filter(path=stripped_path).as_collections():
            yield c

        prefix, _, name = stripped_path.rpartition('/')
        for i in DBItem.objects.filter(collection__path=prefix, name=name).as_items():
            yield i

        if depth == '0':
            return

        for i in DBItem.objects.filter(collection__path=stripped_path).as_items():
            yield i

    def move(self, item, to_collection, to_href):
        try:
            dbcollection = DBCollection.objects.get(path=to_collection._path)
            dbitem = DBItem.objects.get(collection__path=item.collection._path, name=item.href)
        except DBCollection.DoesNotExist:
            pass
        except DBItem.DoesNotExist:
            pass
        else:
            dbitem.collection = dbcollection
            dbitem.name = to_href
            dbitem.save()

    def create_collection(self, href, collection=None, props=None):
        stripped_path = strip_path(href)

        c, created = DBCollection.objects.get_or_create(
            path=stripped_path,
            parent_path=os.path.dirname(stripped_path))

        return c.as_collection()

    @types.contextmanager
    def acquire_lock(self, mode, user):
        with transaction.atomic():
            yield



#!/usr/bin/env python3
import os
from setuptools import setup


# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

data = {
    'name': 'djradicale',
    'version': '0.0.16',
    'author': 'Kyoken',
    'author_email': 'kyoken@kyoken.ninja',
    'description': (
        'Radicale is a free and open-source CalDAV and CardDAV server.'
        'DjRadicale is an Django Application for integration Radicale '
        'with a Django.'),
    'license': 'GPLv3',
    'keywords': 'django radicale carddav caldav',
    'url': 'https://github.com/kyokenn/djradicale',
    'packages': [
        'djradicale',
        'djradicale.migrations',
        'djradicale.tests',
    ],
    'long_description': '',
    'classifiers': [
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Framework :: Django',
    ],
    'install_requires': [
        'Django >= 4.0.1',
        'Radicale >= 3.1.2',
    ],
}

setup(**data)



# Copyright (C) 2022 Kyoken, kyoken@kyoken.ninja

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from django.conf import settings

from radicale.rights import BaseRights
from radicale import pathutils


class Rights(BaseRights):
    def authorization(self, user, path):
        """
        Permissions:

        - R: read collections (excluding address books and calendars)
        - r: read address book and calendar collections
        - i: subset of **r** that only allows direct access via HTTP method GET
             (CalDAV/CardDAV is susceptible to expensive search requests)
        - W: write collections (excluding address books and calendars)
        - w: write address book and calendar collections
        """
        # anonymous is forbidden
        if not user:
            return ''

        sane_path = pathutils.strip_path(path)

        if not sane_path:
            return "R"

        if user != sane_path.split("/", maxsplit=1)[0]:
            return ""
        
        # access to root
        if "/" not in sane_path:
            return 'RW'

        # read+write access to owned collections
        if sane_path.count("/") == 1:
            return 'rw'

        # anyone else is forbidden
        return ""



# Copyright (C) 2016 Okami, okami@fuzetsu.info

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from django.test import TestCase

# from radicale import ical

# from ..storage.django import ICAL_TYPES


class SerializationTestCase(TestCase):
    DATA_CASES = ((
        '''BEGIN:VCALENDAR
CALSCALE:GREGORIAN
PRODID:-//Ximian//NONSGML Evolution Calendar//EN
VERSION:2.0
BEGIN:VTIMEZONE
TZID:/freeassociation.sourceforge.net/Europe/Moscow
X-LIC-LOCATION:Europe/Moscow
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19300621T000000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19810401T000000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19811001T000000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19820401T000000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19821001T000000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19830401T000000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19831001T000000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19840401T000000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19840930T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19850331T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19850929T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19860330T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19860928T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19870329T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19870927T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19880327T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19880925T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19890326T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19890924T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19900325T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19900930T030000
TZOFFSETFROM:+0300
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:EEST
DTSTART:19910331T020000
TZOFFSETFROM:+0200
TZOFFSETTO:+0300
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:EET
DTSTART:19910929T030000
TZOFFSETFROM:+0300
TZOFFSETTO:+0200
END:STANDARD
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19920119T020000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19920329T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19920927T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19930328T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19930926T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19940327T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19940925T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19950326T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19950924T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19960331T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19961027T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19970330T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19971026T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19980329T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19981025T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19990328T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19991031T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:20000326T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20001029T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:20010325T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20011028T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:20020331T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20021027T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:20030330T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20031026T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:20040328T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20041031T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:20050327T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20051030T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:20060326T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20061029T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:20070325T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20071028T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:20080330T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20081026T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:20090329T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20091025T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:20100328T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20101031T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20110327T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
END:STANDARD
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20141026T020000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
END:STANDARD
END:VTIMEZONE
BEGIN:VEVENT
UID:20160729T101032Z-9313-1000-4291-10@localhost
DTSTAMP:20160729T063824Z
DTSTART;TZID=/freeassociation.sourceforge.net/Europe/Moscow:
 20160721T090000
DTEND;TZID=/freeassociation.sourceforge.net/Europe/Moscow:
 20160721T093000
SEQUENCE:2
SUMMARY:zzzzz
CLASS:PUBLIC
TRANSP:OPAQUE
CREATED:20160729T101116Z
LAST-MODIFIED:20160729T101116Z
END:VEVENT
END:VCALENDAR''',

        '''BEGIN:VCALENDAR
PRODID:-//Radicale//NONSGML Radicale Server//EN
VERSION:2.0
BEGIN:VTIMEZONE
TZID:/freeassociation.sourceforge.net/Europe/Moscow
X-LIC-LOCATION:Europe/Moscow
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19300621T000000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19810401T000000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19811001T000000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19820401T000000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19821001T000000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19830401T000000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19831001T000000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19840401T000000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19840930T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19850331T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19850929T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19860330T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19860928T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19870329T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19870927T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19880327T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19880925T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19890326T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19890924T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19900325T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19900930T030000
TZOFFSETFROM:+0300
TZOFFSETTO:+0300
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:EEST
DTSTART:19910331T020000
TZOFFSETFROM:+0200
TZOFFSETTO:+0300
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:EET
DTSTART:19910929T030000
TZOFFSETFROM:+0300
TZOFFSETTO:+0200
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19920119T020000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19920329T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19920927T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19930328T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19930926T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19940327T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19940925T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19950326T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19950924T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19960331T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19961027T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19970330T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19971026T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19980329T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19981025T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:19990328T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:19991031T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:20000326T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20001029T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:20010325T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20011028T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:20020331T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20021027T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:20030330T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20031026T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:20040328T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20041031T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:20050327T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20051030T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:20060326T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20061029T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:20070325T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20071028T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:20080330T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20081026T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:20090329T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20091025T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:MSD
DTSTART:20100328T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20101031T030000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20110327T020000
TZOFFSETFROM:+0300
TZOFFSETTO:+0400
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
BEGIN:STANDARD
TZNAME:MSK
DTSTART:20141026T020000
TZOFFSETFROM:+0400
TZOFFSETTO:+0300
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:STANDARD
X-RADICALE-NAME:/freeassociation.sourceforge.net/Europe/Moscow
END:VTIMEZONE
BEGIN:VEVENT
UID:20160729T101032Z-9313-1000-4291-10@localhost
DTSTAMP:20160729T063824Z
DTSTART;TZID=/freeassociation.sourceforge.net/Europe/Moscow:
 20160721T090000
DTEND;TZID=/freeassociation.sourceforge.net/Europe/Moscow:
 20160721T093000
SEQUENCE:2
SUMMARY:zzzzz
CLASS:PUBLIC
TRANSP:OPAQUE
CREATED:20160729T101116Z
LAST-MODIFIED:20160729T101116Z
X-RADICALE-NAME:20160729T101032Z-9313-1000-4291-10_localhost-20160729T101116Z.ics
END:VEVENT
END:VCALENDAR
'''),
        )

    # HEADERS = (
    #     ical.Header('PRODID:-//Radicale//NONSGML Radicale Server//EN'),
    #     ical.Header('VERSION:2.0'))
    TAG = 'VCALENDAR'

    maxDiff = None

    # def test_serialization(self):
    #     name = '20160729T101032Z-9313-1000-4291-10_localhost-20160729T101116Z.ics'

    #     for input_text, stored_text in self.DATA_CASES:
    #         items = ical.Collection._parse(input_text, ICAL_TYPES, name)
    #         self.assertEqual(name in items, True)

    #         timezones = list(filter(
    #             lambda x: x.tag == ical.Timezone.tag, items.values()))
    #         serialized_text = ical.serialize(
    #             self.TAG, self.HEADERS, [items[name]] + timezones)
    #         self.assertEqual(serialized_text, stored_text)

    #         items2 = ical.Collection._parse(serialized_text, ICAL_TYPES, name)
    #         self.assertEqual(name in items2, True)
